==> num-java-client-lib-0.0.13-sources.jar: uk/num/numlib/internal/module/ModuleFactory.java <==
/*
 * Copyright (c) 2019. NUM Technology Ltd
 */

package uk.num.numlib.internal.module;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import uk.num.numlib.exc.NumBadURLException;
import uk.num.numlib.exc.NumInvalidParameterException;
import uk.num.numlib.internal.ctx.AppContext;
import uk.num.numlib.internal.util.SimpleCache;

/**
 * A factory for ModuleDNSQuery objects.
 *
 * @author tonywalmsley
 */
public class ModuleFactory {
    private static final Logger LOG = LoggerFactory.getLogger(ModuleFactory.class);
    /**
     * A cache for module/domain name combinations.
     */
    private final SimpleCache<String, ModuleDNSQueries> moduleMap = new SimpleCache<>();

    /**
     * Create and initialise a ModuleDNSQueries object or use a cached object.
     *
     * @param appContext the AppContext
     * @param moduleName the module name string, e.g. "1"
     * @param domainName the domain name, URL, or email address to be queried for a NUM record.
     * @return a ModuleDNSQueries object
     * @throws NumInvalidParameterException on error
     * @throws NumBadURLException           on error
     */
    public ModuleDNSQueries getInstance(final AppContext appContext, final String moduleName, final String domainName) throws
                                                                                                                       NumInvalidParameterException,
                                                                                                                       NumBadURLException {
        if (moduleName == null || moduleName.trim()
                .isEmpty()) {
            LOG.error("Invalid moduleName supplied: null or empty.");
            throw new NumInvalidParameterException("Invalid moduleName supplied: null or empty.");
        }
        if (domainName == null || domainName.trim()
                .isEmpty()) {
            LOG.error("Invalid domainName supplied: null or empty.");
            throw new NumInvalidParameterException("Invalid domainName supplied: null or empty.");
        }

        ModuleDNSQueries result;

        final String key = moduleName + "_" + domainName;
        // Critical section - we're reading then updating moduleMap, which is a potential race condition
        synchronized (moduleMap) {
            result = moduleMap.get(key);
            if (result == null) {
                result = new ModuleDNSQueries(moduleName, domainName);

                // Initialisation as a separate step since its an 'expensive' operation. Allows us to create lots of
                // Modules if necessary but then only initialise the ones we use.
                result.initialise(appContext);

                // Do this last in case there's an exception so we don't store an invalid ModuleDNSQueries object
                moduleMap.put(key, result);
                LOG.trace("Cached a new set of ModuleDNSQueries.");
            } else {
                LOG.trace("Using cached ModuleDNSQueries.");
            }
        }
        return result;
    }
}

==> num-java-client-lib-0.0.21-sources.jar: uk/num/numlib/internal/module/ModuleFactory.java <==
/*
 * Copyright (c) 2019. NUM Technology Ltd
 */

package uk.num.numlib.internal.module;

import lombok.extern.log4j.Log4j2;
import uk.num.numlib.exc.NumBadURLException;
import uk.num.numlib.exc.NumInvalidParameterException;
import uk.num.numlib.internal.ctx.AppContext;
import uk.num.numlib.internal.util.NonBlankString;
import uk.num.numlib.internal.util.SimpleCache;

/**
 * A factory for ModuleDNSQuery objects.
 *
 * @author tonywalmsley
 */
@Log4j2
public class ModuleFactory {
    /**
     * A cache for module/NUM ID combinations.
     */
    private final SimpleCache<String, ModuleDNSQueries> moduleMap = new SimpleCache<>();

    /**
     * Create and initialise a ModuleDNSQueries object or use a cached object.
     *
     * @param appContext the AppContext
     * @param moduleName the module name string, e.g. "1"
     * @param numId      the NUM ID to be queried for a NUM record.
     * @return a ModuleDNSQueries object
     * @throws NumInvalidParameterException on error
     * @throws NumBadURLException           on error
     */
    public ModuleDNSQueries getInstance(final AppContext appContext, final NonBlankString moduleName, final NonBlankString numId) throws
                                                                                                                                  NumInvalidParameterException,
                                                                                                                                  NumBadURLException {
        ModuleDNSQueries result;

        final String key = moduleName.value + "_" + numId.value;
        // Critical section - we're reading then updating moduleMap, which is a potential race condition
        synchronized (moduleMap) {
            result = moduleMap.get(key);
            if (result == null) {
                result = new ModuleDNSQueries(moduleName, numId);

                // Initialisation as a separate step since its an 'expensive' operation. Allows us to create lots of
                // Modules if necessary but then only initialise the ones we use.
                result.initialise(appContext);

                // Do this last in case there's an exception so we don't store an invalid ModuleDNSQueries object
                moduleMap.put(key, result);
                log.trace("Cached a new set of ModuleDNSQueries.");
            } else {
                log.trace("Using cached ModuleDNSQueries.");
            }
        }
        return result;
    }
}
